// AUTOGENERATED FILE - DO NOT EDIT

var pt = window.PropTypes;
var pk = window['vizro_dash_components'];

pk.Cascader.propTypes = {id:pt.any,
 options:pt.any,
 value:pt.any,
 multi:pt.any,
 searchable:pt.any,
 clearable:pt.any,
 placeholder:pt.any,
 disabled:pt.any,
 maxHeight:pt.any,
 className:pt.any,
 style:pt.any,
 optionHeight:pt.any,
 debounce:pt.any,
 closeOnSelect:pt.any,
 search_value:pt.any,
 labels:pt.any,
 persistence:pt.any,
 persisted_props:pt.any,
 persistence_type:pt.any};
pk.Markdown.propTypes = {id:pt.any,
 className:pt.any,
 mathjax:pt.any,
 dangerously_allow_html:pt.any,
 link_target:pt.any,
 children:pt.any,
 dedent:pt.any,
 highlight_config:pt.any,
 style:pt.any};
